# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['understory',
 'understory.braid',
 'understory.data',
 'understory.data.templates',
 'understory.editor',
 'understory.editor.templates',
 'understory.indieauth',
 'understory.indieauth.client',
 'understory.indieauth.client.templates',
 'understory.indieauth.server',
 'understory.indieauth.server.templates',
 'understory.jobs',
 'understory.jobs.templates',
 'understory.micropub',
 'understory.micropub.content',
 'understory.micropub.content.templates',
 'understory.micropub.media',
 'understory.micropub.media.templates',
 'understory.micropub.posts',
 'understory.micropub.posts.templates',
 'understory.microsub',
 'understory.microsub.templates',
 'understory.owner',
 'understory.owner.templates',
 'understory.providers',
 'understory.providers.templates',
 'understory.search',
 'understory.search.templates',
 'understory.sites',
 'understory.sites.templates',
 'understory.system',
 'understory.system.templates',
 'understory.tracker',
 'understory.tracker.templates',
 'understory.webmention',
 'understory.webmention.templates',
 'understory.webmention.templates.received',
 'understory.webmention.templates.sent',
 'understory.websub',
 'understory.websub.templates',
 'understory.websub.templates.received',
 'understory.websub.templates.sent',
 'web',
 'web.framework',
 'web.framework.templates',
 'web.headers',
 'web.host',
 'web.host.templates',
 'web.markdown',
 'web.response',
 'web.templating',
 'web.uri']

package_data = \
{'': ['*'],
 'understory.system': ['static/*'],
 'web.framework': ['static/braid.js',
                   'static/braid.js',
                   'static/braid.js',
                   'static/braid.js',
                   'static/braid.js',
                   'static/orchid.js',
                   'static/orchid.js',
                   'static/orchid.js',
                   'static/orchid.js',
                   'static/orchid.js',
                   'static/roots.js',
                   'static/roots.js',
                   'static/roots.js',
                   'static/roots.js',
                   'static/roots.js',
                   'static/solarized.css',
                   'static/solarized.css',
                   'static/solarized.css',
                   'static/solarized.css',
                   'static/solarized.css',
                   'static/understory.js',
                   'static/understory.js',
                   'static/understory.js',
                   'static/understory.js',
                   'static/understory.js']}

install_requires = \
['Pygments>=2.11.2,<3.0.0',
 'Unidecode>=1.2.0,<2.0.0',
 'acme-tiny>=4.1.0,<5.0.0',
 'certifi>=2021.10.8,<2022.0.0',
 'clicks>=0.0.47,<0.0.48',
 'cssselect>=1.1.0,<2.0.0',
 'dnspython>=2.1.0,<3.0.0',
 'emoji>=1.2.0,<2.0.0',
 'gevent>=21.1.2,<22.0.0',
 'gunicorn>=20.1.0,<21.0.0',
 'hstspreload>=2021.12.1,<2022.0.0',
 'httpagentparser>=1.9.1,<2.0.0',
 'jsonpatch>=1.32,<2.0',
 'lxml>=4.8.0,<5.0.0',
 'mf2py>=1.1.2,<2.0.0',
 'mf2util>=0.5.1,<0.6.0',
 'mimeparse>=0.1.3,<0.2.0',
 'pendulum>=2.1.2,<3.0.0',
 'pillow>=9.0.1,<10.0.0',
 'pycryptodome>=3.10.1,<4.0.0',
 'pyscreenshot>=3.0,<4.0',
 'pyvirtualdisplay>=3.0,<4.0',
 'regex>=2021.7.6,<2022.0.0',
 'requests[socks]>=2.27.1,<3.0.0',
 'selenium>=4.1.2,<5.0.0',
 'semver>=2.13.0,<3.0.0',
 'slrzd>=0.0.5,<0.0.6',
 'sqlyte>=0.0.10,<0.0.11',
 'waitress>=2.0.0,<3.0.0',
 'watchdog>=2.1.3,<3.0.0']

entry_points = \
{'console_scripts': ['web = web.__main__:main'],
 'understory': ['braid = understory.braid:app',
                'data = understory.data:app',
                'editor = understory.editor:app',
                'indieauth_client = understory.indieauth.client:app',
                'indieauth_server = understory.indieauth.server:app',
                'jobs = understory.jobs:app',
                'micropub_content = understory.micropub.content:app',
                'micropub_media = understory.micropub.media:app',
                'micropub_server = understory.micropub.posts:app',
                'microsub_server = understory.microsub:app',
                'owner = understory.owner:app',
                'providers = understory.providers:app',
                'search = understory.search:app',
                'sites = understory.sites:app',
                'system = understory.system:app',
                'tracker = understory.tracker:app',
                'webmention_endpoint = understory.webmention:app',
                'websub_endpoint = understory.websub:app']}

setup_kwargs = {
    'name': 'understory',
    'version': '0.0.153',
    'description': 'Social web framework',
    'long_description': '# Understory\nSocial web framework\n\n## Create a web app\n\nYou should use `Poetry`.\n\n```shell\npoetry init\npoetry add understory\n```\n\nCreate the following file structure:\n\n```\nexample\n├── __init__.py\n├── static\n│   └── screen.css\n└── templates\n    ├── __init__.py\n    └── index.html\n```\n\n**example/\\_\\_init\\_\\_.py**\n\n```python\nimport web\nfrom understory import indieauth\n\napp = web.application(__name__, mounts=[indieauth.client.app])\n\n\n@app.control("")\nclass Landing:\n    def get(self):\n        return app.view.index(web.tx.user)\n```\n\n**example/static/screen.css**\n\n```css\np.greeting { color: #f00; font-size: 5em; }\n```\n\n**example/templates/\\_\\_init\\_\\_.py**\n\n```python\nimport random\n\nfrom understory.indieauth import web_sign_in\n\n__all__ = ["random", "web_sign_in_form"]\n```\n\n**example/templates/index.html**\n\n```html\n$def with (user, greeting)\n\n$if user.session:\n    $ greeting = random.choice(["hello", "hi", "howdy"])\n    <p class=greeting>$greeting $user.name ($user.url)</p>\n$else:\n    $:web_sign_in_form()\n```\n\n### Serve it locally\n\n```shell\npoetry run web serve example\n```\n\nThe server will automatically reload on changes to the source code.\n\n### Host it in the cloud\n\n```shell\npoetry run web host example\n```\n\n## URL parsing\n\nDefaults to safe-mode and raises DangerousURL eagerly. Up-to-date public\nsuffix and HSTS support.\n\n    >>> url = skutterbug.uri("example.cnpy.gdn/foo/bar?id=38")\n    >>> url.host\n    \'example.cnpy.gdn\'\n    >>> url.suffix\n    \'cnpy.gdn\'\n    >>> url.is_hsts()\n    True\n\n## Cache\n\nuses SQLite\n\n    >>> cache = skutterbug.cache()\n    >>> cache["indieweb.org/note"].entry["summary"]\n    \'A note is a post that is typically short unstructured* plain text, written & posted quickly, that has its own permalink page.\'\n    >>> cache["indieweb.org/note"].entry["summary"]  # served from cache\n    \'A note is a post that is typically short unstructured* plain text, written & posted quickly, that has its own permalink page.\'\n\n### Microformat parsing\n\nParse `mf2` from `HTML`. Analyze vocabularies for stability/interoperability.\n\n## Browser\n\nuses Firefox via Selenium\n\n    >>> # browser = skutterbug.Firefox()\n    >>> # browser.go("en.wikipedia.org/wiki/Pasta")\n    >>> # browser.shot("wikipedia-pasta.png")\n',
    'author': 'Angelo Gladding',
    'author_email': 'angelo@ragt.ag',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
